@@ -21,11 +21,13 @@
 
 gvisChart <- function(type, checked.data, options, chartid, package, formats = NULL){
   
-  Chart = gvis(type=type, checked.data, options=options, chartid=chartid, package, formats=formats)
+  Chart = gvis(type=type, checked.data, options=options, 
+               chartid=chartid, package, formats=formats)
   chartid <- Chart$chartid
   htmlChart <- Chart$chart
   
-  htmlScaffold <- gvisHtmlWrapper(title="", chartid=chartid, dataName=options$dataName,
+  htmlScaffold <- gvisHtmlWrapper(title="", chartid=chartid, 
+                                  dataName=options$dataName,
                                   type=tolower(type))
   
   output <- structure(
@@ -63,7 +65,7 @@ gvis <- function(type="", data, options, chartid, package, formats=NULL){
   data.json <- output$json
   
   ## check for not allowed data types
-  checkTypes <- data.type %in% options$data$allowed 
+  checkTypes <- data.type %in% options$data$allowed | isRoleColumn(data.type)
   if(sum(!checkTypes)){
     message <- paste("Only the following data types are allowed: ", 
                      paste(options$data$allowed, collapse=", "), "\n",
@@ -112,8 +114,29 @@ return(data);
 '
   jsData <- sprintf(jsData, chartid,
                     data.json,
-                    paste(paste("data.addColumn('", data.type, "','",
-                                names(data.type), "');", sep=""), collapse="\n"))
+                    paste(sapply(1:length(data.type), function(x)
+                      if (endsIn(names(data.type)[x], '.tooltip'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'tooltip', 'p': {'html': true}});", sep="")
+                      else if(endsIn(names(data.type)[x], '.interval'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'interval'});", sep="")
+                      else if(endsIn(names(data.type)[x], '.style'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'style'});", sep="")                                 
+                      else if(endsIn(names(data.type)[x], '.annotation'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'annotation'});", sep="")
+                      else if(endsIn(names(data.type)[x], '.annotationText'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'annotationText'});", sep="")
+                      else if(endsIn(names(data.type)[x], '.certainty'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'certainty'});", sep="")
+                      else if(endsIn(names(data.type)[x], '.scope'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'scope'});", sep="")
+                      else if(endsIn(names(data.type)[x], '.emphasis'))
+                        paste("data.addColumn({type: '", data.type[x], "', role: 'emphasis'});", sep="")
+                      else
+                        paste("data.addColumn('", data.type[x], "','",
+                              names(data.type)[x], "');", sep="")
+                    ), collapse="\n")
+                    
+  )
   
   jsDisplayChart <- '
 // jsDisplayChart
@@ -329,7 +352,22 @@ fixBackslash <- function(x){
   return(x)
 }
 
+endsIn <- function(source, target){
+  substr(source, nchar(source)-nchar(target)+1, nchar(source)) %in% target
+}
 
+isRoleColumn <- function(x) {
+  return (
+    (x %in% c("string") & endsIn(names(x), ".tooltip")) |
+      (x %in% c("string") & endsIn(names(x), ".annotation")) |
+      (x %in% c("string") & endsIn(names(x), ".style")) |
+      (x %in% c("string") & endsIn(names(x), ".annotationText")) |
+      (x %in% c("boolean") & endsIn(names(x), ".certainty")) |
+      (x %in% c("boolean") & endsIn(names(x), ".emphasis")) |
+      (x %in% c("boolean") & endsIn(names(x), ".scope")) |
+      (x %in% c("number") & endsIn(names(x), ".interval"))
+  )
+}
 check.location <- function(x){
   y = as.character(x)
   if (! is.character(y))
@@ -525,7 +563,7 @@ body {
   htmlFooter <- '
 <!-- htmlFooter -->
 <span> 
-  %s &#8226; <a href="http://code.google.com/p/google-motion-charts-with-r/">googleVis-%s</a>
+  %s &#8226; <a href="https://github.com/mages/googleVis">googleVis-%s</a>
   &#8226; <a href="https://developers.google.com/terms/">Google Terms of Use</a> &#8226; %s
 </span></div>
 </body>
@@ -535,7 +573,7 @@ body {
   if(type %in% "gvisMerge"){
     policy <- "Data Policy: See individual charts"
   }else{
-    policy <- sprintf('<a href="https://google-developers.appspot.com/chart/interactive/docs/gallery/%s.html#Data_Policy">Data Policy</a>', type)
+    policy <- sprintf('<a href="https://google-developers.appspot.com/chart/interactive/docs/gallery/%s#Data_Policy">Data Policy</a>', type)
   }
   
   htmlFooter <- sprintf(htmlFooter, R.Version()$version.string,
@@ -610,4 +648,4 @@ as.string <- function(x,file="",append=FALSE) {
 
 is.string <- function(x) {
   return(class(x)=="string")
-}
\ No newline at end of file
+}
